import numpy as np


def euclidean_distance(x, y):
    _x = np.array(x)
    _y = np.array(y)
    return np.sqrt(np.sum((_x - _y) ** 2, axis=1))


def manhattan_distance(x, y):
    _x = np.array(x)
    _y = np.array(y)
    return np.sum(np.abs(_x - _y))


def hamming_distance(x, y):
    _x = np.array(x)
    _y = np.array(y)
    return np.sum(_x != _y)


def minkowski_distance(x, y, dimension):
    _x = np.array(x)
    _y = np.array(y)
    return np.linalg.norm((_x - _y), ord=dimension)


def weighted_euclidean_distance(x, y, weights):
    _x = np.array(x)
    _y = np.array(y)
    _w = np.array(weights)
    return np.sqrt(np.sum(_w * ((_x - _y) ** 2), axis=1))


def weighted_manhattan_distance(x, y, weights):
    _x = np.array(x)
    _y = np.array(y)
    _w = np.array(weights)
    return np.sum(_w * np.abs(_x - _y))


def weighted_minkowski_distance(x, y, dimension, weights):
    _x = np.array(x)
    _y = np.array(y)
    _w = np.array(weights)
    return np.linalg.norm(_w * (_x - _y), ord=dimension)